/* file: analyze.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions for the analysis panel of WAVE

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include "wave.h"
#include "gwave.h"
#include "interface.h"
#include "support.h"
#include "defaults.h"
#include <stdio.h>

#ifdef __WIN32__
int mkstemp(char *);
#endif

#ifdef __WIN32__
extern HANDLE console_create();
extern void console_input(HANDLE console_pipe, gchar *cmd, gint len);
#else
extern GtkWidget *console_create();
extern void console_input(GtkWidget *term, gchar *cmd, gint len);
#endif

#define MAXLL		1024		/* maximum length of a line in the menu
					   file including continuation lines */

void do_analysis(), edit_menu_file(), set_signal(), set_start(), set_stop(),
  set_back(), set_ahead(), set_siglist(), show_command_window(),
  add_signal_choice(), delete_signal_choice(), create_analysis_menu();

GtkWidget *analyze_frame;
GtkWidget *start_item, *astart_item, *dstart_item, *end_item, *aend_item,
           *dend_item, *signal_item, *signal_name_item, *siglist_item;
GtkWidget *menu_button_table;

int analyze_popup_active = -1;

GSList *menu_head = NULL;
typedef struct WAVEMenuEntry {
  char *command;
  GtkWidget *button;
} WaveMenuEntry;

char *print_command = NULL;
int menu_read = 0;

void print_proc(GtkMenuItem *menuitem, gpointer user_data)
{
  char default_print_command[256];
  void read_menu();
  
  if (menu_read == 0)
    read_menu();
  if (print_command == NULL) {
    sprintf(default_print_command, "echo $RECORD $LEFT-$RIGHT |\
 pschart -a $ANNOTATOR -g -l -L -n 0 -R -t 20 -v 8 - | %s\n", psprint);
    print_command = default_print_command;
  }
  do_command(print_command);
}

char *open_url_command = NULL;

void open_url()
{
  char default_open_url_command[256];
  void read_menu();
  
  if (menu_read == 0)
    read_menu();
  if (open_url_command == NULL) {
    sprintf(default_open_url_command, "urlview $URL\n");
    open_url_command = default_open_url_command;
  }
  do_command(open_url_command);
}


void read_menu()
{
  gchar *wavemenu, ts[RNLMAX+30], linebuf[MAXLL+1], *p, *p2;
  int l;
  FILE *ifile = NULL;
  GSList *mep;
  WaveMenuEntry *entry;

  /* Record that an attempt has been made to read the menu. */
  
  /* Clear the menu entry list. */

  for ( mep = menu_head; mep; mep = mep->next)
    g_free( ((WaveMenuEntry *)(mep->data))->command );
  g_slist_free(menu_head);
  menu_head = NULL;
  
  menu_read = 1;
  
  wavemenu = get_wave_filename( WAVE_MENU_FILE | WAVE_FILE_BEST );
  if ((ifile = fopen(wavemenu, "r")) == NULL) {
    wavemenu = get_wave_filename( WAVE_MENU_FILE | WAVE_FILE_GLOBAL );
    fprintf(stderr, "%s\n", wavemenu);
    if ((ifile = fopen(wavemenu, "r")) == NULL) {
      sprintf(ts, "Could not open menu file, %s.", wavemenu);
      notice_prompt(ts, "Continue", NULL);
      return;
    }
  }
    
  /* Read the menu file. */
  while (fgets(linebuf, MAXLL+1, ifile)) {	/* read the next line */
    while ((l = strlen(linebuf)) > 1 && l < MAXLL && linebuf[l-2]=='\\' &&
	   fgets(&linebuf[l-2], MAXLL+1 - (l-2), ifile))
      /* append continuation line(s), if any */
      if (linebuf[l-2] == '\t' || linebuf[l-2] == ' ') {
	/* discard initial whitespace in continuation lines, if any */
	char *p, *q;
	
	for (p = q = linebuf+l-2; *p == '\t' || *p == ' '; p++)
	  ;
	while (*p)
	  *q++ = *p++;
	*q = '\0';
      }
    
    /* Find the first non-whitespace character (the beginning of the
       button label). */
    for (p = linebuf; *p; p++) {
      if (*p == '#') { *p = '\0'; break; }
      else if (*p != ' ' && *p != '\t') break;
    }

    /* Skip comments and empty lines. */
    if (*p == '\0') continue;
    
    /* Find the first embedded tab (the end of the button label). */
    for (p2 = p; *p2 && *p2 != '\t'; p2++)
      ;
    
    /* Skip lines without an embedded tab. */
    if (*p2 == '\0') continue;
    
    *p2++ = '\0';	/* Replace the embedded tab with a null. */
    
    /* Find the next non-whitespace character (the beginning of the
       command). */
    for ( ; *p2 == '\t' || *p2 == ' '; p2++)
      ;
    
    /* Skip lines without a command. */
    if (*p2 == '\n') continue;

    /* Test for special <Print> command definition. */
    if (strcmp(p, "<Print>") == 0) {
      if ( (p = (char *)malloc((unsigned)(strlen(p2)+1))) ) {
	strcpy(p, p2);
	print_command = p;
      }
      continue;
    }
    
    /* Test for special <Open URL> command definition. */
    if (strcmp(p, "<Open URL>") == 0) {
      if ( (p = (char *)malloc((unsigned)(strlen(p2)+1))) ) {
	strcpy(p, p2);
	open_url_command = p;
      }
      continue;
    }
    
    if (!(entry=(WaveMenuEntry *)g_malloc(sizeof(WaveMenuEntry))) ||
	!(entry->command = (char *)g_malloc((unsigned)(strlen(p2)+1)))) {
      notice_prompt("Out of memory while reading menu file","Continue", NULL);
      break;
    }

    entry->button = gtk_button_new_with_label(p);
    g_snprintf( entry->command, strlen(p2)+1, "%s\n", p2 );
    gtk_signal_connect( GTK_OBJECT (entry->button), "clicked",
			GTK_SIGNAL_FUNC (do_analysis), entry->command );
    if ( menu_head )
      mep = g_slist_append( mep, entry ); 
    else
      menu_head = mep = g_slist_append( menu_head, entry );
  }
  fclose(ifile);
  //free(wavemenu);
}

int xaf = -1, yaf = -1;

/* Set up analyze menu and terminal emulator windows. */
void create_analyze_popup()
{
  void recreate_analyze_popup(), reload();
  GtkWidget *box;

  if (menu_read == 0)
    read_menu();  
  analyze_popup_active = 0;

  analyze_frame = create_analyze ();

  start_item = lookup_widget( GTK_WIDGET(analyze_frame), "start_entry");
  end_item = lookup_widget( GTK_WIDGET(analyze_frame), "end_entry");

  signal_item = lookup_widget( GTK_WIDGET(analyze_frame), "signal_spinbutton");
  gtk_spin_button_set_adjustment( GTK_SPIN_BUTTON(signal_item), 
				  GTK_ADJUSTMENT(gtk_adjustment_new(signal_choice, 
						 0, nsig-1, 1, 1, 1)) );
  
  signal_name_item = lookup_widget( GTK_WIDGET(analyze_frame), "signal_name_label");
  gtk_label_set_text( GTK_LABEL( signal_name_item ), signame[signal_choice]);

  astart_item = lookup_widget( GTK_WIDGET(analyze_frame), "astart_entry");
  dstart_item = lookup_widget( GTK_WIDGET(analyze_frame), "dstart_entry");
  reset_start();

  aend_item = lookup_widget( GTK_WIDGET(analyze_frame), "aend_entry");
  dend_item = lookup_widget( GTK_WIDGET(analyze_frame), "dend_entry");
  reset_stop();

  siglist_item = lookup_widget( GTK_WIDGET(analyze_frame), "siglist_entry");
  reset_siglist();
  
  create_analysis_menu();
  box = lookup_widget( GTK_WIDGET (analyze_frame), "analyze_menu_frame");
  gtk_container_add( GTK_CONTAINER (box), GTK_WIDGET(menu_button_table) );

  if (console == NULL)
    console = console_create();
  
}

void
create_analysis_menu() {
  gint i, rows, cols, r, c;
  GSList *mep;

  cols = 6;
  rows = g_slist_length( menu_head ) / cols + 1;
  
  if ( menu_button_table )
    gtk_widget_destroy( GTK_WIDGET(menu_button_table) );
  menu_button_table = gtk_table_new(rows, cols, FALSE);  

  for (i=0, mep = menu_head; mep; mep = mep->next, i++ ) {
    r = i / cols;
    c = i - r*cols;
    gtk_table_attach( GTK_TABLE (menu_button_table), 
		      GTK_WIDGET (((WaveMenuEntry *)(mep->data))->button),
		      c, c+1, r, r+1,
		      GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 5, 5);
    gtk_widget_show( GTK_WIDGET (((WaveMenuEntry *)(mep->data))->button) );
  }
  gtk_widget_show( GTK_WIDGET(menu_button_table) );

}

void dismiss_analyze_popup(GtkButton *button, gpointer user_data)
{
  gtk_widget_hide(analyze_frame);
}

gboolean on_analyze_delete_event(GtkWidget *widget, 
				 GdkEvent *event, gpointer user_data)
{
  dismiss_analyze_popup(NULL,NULL);
  dismiss_analysis_cmds(NULL,NULL);
  return TRUE;
}


/* Recreate analyze popup (if reread menu button was selected). */
void
recreate_analyze_popup(GtkButton *button,
		       gpointer user_data)
{
  GtkWidget *box;
  if (analyze_frame) {
    read_menu();
    create_analysis_menu();
    box = lookup_widget( GTK_WIDGET (analyze_frame), "analyze_menu_frame");
    gtk_container_add( GTK_CONTAINER (box), GTK_WIDGET(menu_button_table) );
  }
  else
    create_analyze_popup();
}

/* Make the analysis menu window appear. */
void analyze_proc(GtkMenuItem *menuitem, gpointer user_data)
{
  if (analyze_popup_active < 0) create_analyze_popup();

  show_command_window(NULL,NULL);
  gtk_widget_show(analyze_frame);
  analyze_popup_active = 1;
}

/* Edit the menu file. */
void
edit_menu_file                         (GtkButton       *button,
                                        gpointer         user_data)
{
  gchar *wavemenu_user, *wavemenu_global = NULL;
  GString *edit_command, *editor;

  int result;

  

  edit_command = g_string_new("");  
  editor = g_string_new( g_getenv("EDITOR") ); 
  if ( editor->len < 1 )
    g_string_assign( editor, EDITOR ); 
  
  wavemenu_user = get_wave_filename( WAVE_MENU_FILE | WAVE_FILE_USER );
  
  if (wavemenu_user == NULL) {

    result = notice_prompt("You are now using a read only menu file, \
which you may not edit\ndirectly. Press `Copy' to copy the system default menufile \
into your home wave directory as `wavemenu.def'\n- or -\nPress `Cancel' if \
you prefer not to edit a menu file.",
			   "Copy", "Cancel");
    
    if (result != NOTICE_YES)
      return;

    wavemenu_global = get_wave_filename( WAVE_MENU_FILE | WAVE_FILE_GLOBAL );
    wavemenu_user = get_wave_filename( WAVE_MENU_FILE | WAVE_FILE_USER | WAVE_FILE_EDITABLE );

    g_string_sprintf(edit_command, "cp %s %s\n", wavemenu_global, wavemenu_user);
    do_command(edit_command->str);
  }

  g_string_sprintf(edit_command, "%s %s\n", editor->str, wavemenu_user);
  show_command_window(NULL,NULL);
  do_command(edit_command->str);
  g_string_free(editor, TRUE);
  g_string_free(edit_command, TRUE);
}

/* Set variables needed for analysis routines. */
void set_signal(GtkEditable *editable, gpointer user_data)
{
  signal_choice = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(signal_item));
  gtk_label_set_text( GTK_LABEL(signal_name_item), signame[signal_choice]);
  sig_highlight(signal_choice);
}

void set_signal_choice(i)
int i;
{
  int j=1;
  
  if (sig_mode == 0)
    j = i;
  else if (0 <= i && i < siglistlen)
    j = siglist[i];
  if (0 <= j && j < nsig) {
    signal_choice = j;
    if (analyze_popup_active >= 0) {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(signal_item), signal_choice);
      gtk_label_set_text( GTK_LABEL(signal_name_item), signame[signal_choice]);
    }
    sig_highlight(signal_choice);
  }
}

void set_siglist(GtkEditable *editable, gpointer user_data)
{
  char *s = gtk_entry_get_text(GTK_ENTRY(siglist_item));
  
  siglistlen = 0;
  while (*s == ' ' || *s == '\t')
    s++;
  while (siglistlen < WFDB_MAXSIG && *s != '\0' &&
	 sscanf(s, "%d", &siglist[siglistlen]) == 1) {
    if (0 <= siglist[siglistlen] && siglist[siglistlen] < nsig)
      siglistlen++;
    while (*s != ' ' && *s != '\t') {
      if (*s == '\0') {
	reset_siglist();
	return;
      }
      else s++;
    }
    while (*s == ' ' || *s == '\t')
      s++;
  }
  reset_siglist();
}

void set_start(GtkWidget *widget, gpointer user_data)
{
  struct ap *a;
  char *start_string=NULL, *astart_string=NULL, 
    *dstart_string=NULL, *p=NULL, buf[30];
  int i;
  WFDB_Time t=0;

  i = GPOINTER_TO_INT(user_data);
  if ( (a = get_ap()) ) {
    int redraw;
    
    redraw = (display_start_time <= begin_analysis_time &&
	      begin_analysis_time < display_start_time + nsamp);
    switch (i) {
      case 'e':
	start_string = gtk_entry_get_text(GTK_ENTRY(start_item));
	t = strtim(start_string);
	p = mstimstr(-t);
	if (*p == '[') {
	  *(p+13) = *(p+24) =  '\0';
	  gtk_entry_set_text(GTK_ENTRY(astart_item), p+1);
	  gtk_entry_set_text(GTK_ENTRY(dstart_item), p+14);
	}
	else {
	  gtk_editable_delete_text( GTK_EDITABLE(astart_item), 0, -1);
	  gtk_editable_delete_text( GTK_EDITABLE(dstart_item), 0, -1);
	}
	break;
      case 'a':
	astart_string = gtk_entry_get_text( GTK_ENTRY(astart_item) );
	astart_string = gtk_entry_get_text( GTK_ENTRY(dstart_item) );
	sprintf(buf, "[%s %s]", astart_string, dstart_string);
	if ((t = -strtim(buf)) <= 0L) {
	  /* tried to set a time before the beginning of the record */
	  t = 0L;	/* go to the beginning instead */
	  gtk_entry_set_text( GTK_ENTRY(start_item), "beggining" );
	}
	else
	  gtk_entry_set_text( GTK_ENTRY(start_item), mstimstr(t) );
	set_start(start_item, NULL);
	break;
      case 'd':
	dstart_string = gtk_entry_get_text( GTK_ENTRY(dstart_item) );
	/* changed the date -- try to go to midnight on that date */
	sprintf(buf, "[0:0:0 %s]", dstart_string);
	if ((t = -strtim(buf)) <= 0L) {
	  /* tried to set a time before the beginning of the record */
	  t = 0L;	/* go to the beginning instead */
	  gtk_entry_set_text( GTK_ENTRY(start_item), "beggining" );
	}
	else
	  gtk_entry_set_text( GTK_ENTRY(start_item), mstimstr(t) );
	set_start(start_item, NULL);
	break;
    }
    
    a->this.anntyp = BEGIN_ANALYSIS;
    a->this.subtyp = a->this.num = 0;
    a->this.chan = 127;
    a->this.aux = NULL;
    a->this.time = t;
    insert_annotation(a);
    if (redraw || (display_start_time <= begin_analysis_time &&
		   begin_analysis_time < display_start_time + nsamp)) {
      clear_annotation_display();
      show_annotations(display_start_time, nsamp);
    }
  }
}

void set_stop(GtkWidget *widget, gpointer user_data)
{
  struct ap *a;
  char *end_string=NULL, *aend_string=NULL, *dend_string=NULL, *p=NULL, buf[30];
  int i;
  WFDB_Time t=0;

  i = GPOINTER_TO_INT( user_data );
  if ( (a = get_ap()) ) {
    int redraw;

    redraw = (display_start_time <= end_analysis_time &&
	      end_analysis_time < display_start_time + nsamp);
    switch (i) {
      case 'e':
	end_string = gtk_entry_get_text(GTK_ENTRY(end_item));
	t = strtim(end_string);
	p = mstimstr(-t);
	if (*p == '[') {
	  *(p+13) = *(p+24) =  '\0';
	  gtk_entry_set_text(GTK_ENTRY(aend_item), p+1);
	  gtk_entry_set_text(GTK_ENTRY(dend_item), p+14);
	}
	else {
	  gtk_editable_delete_text( GTK_EDITABLE(aend_item), 0, -1);
	  gtk_editable_delete_text( GTK_EDITABLE(dend_item), 0, -1);
	}
	break;
      case 'a':
	aend_string = gtk_entry_get_text( GTK_ENTRY(aend_item) );
	aend_string = gtk_entry_get_text( GTK_ENTRY(dend_item) );
	sprintf(buf, "[%s %s]", aend_string, dend_string);
	if ((t = -strtim(buf)) <= 0L) {
	  /* tried to set a time before the beginning of the record */
	  t = 0L;	/* go to the beginning instead */
	  gtk_entry_set_text( GTK_ENTRY(end_item), "beggining" );
	}
	else
	  gtk_entry_set_text( GTK_ENTRY(end_item), mstimstr(t) );
	set_stop(end_item, NULL);
	break;
      case 'd':
	dend_string = gtk_entry_get_text( GTK_ENTRY(dend_item) );
	/* changed the date -- try to go to midnight on that date */
	sprintf(buf, "[0:0:0 %s]", dend_string);
	if ((t = -strtim(buf)) <= 0L) {
	  /* tried to set a time before the beginning of the record */
	  t = 0L;	/* go to the beginning instead */
	  gtk_entry_set_text( GTK_ENTRY(end_item), "beggining" );
	}
	else
	  gtk_entry_set_text( GTK_ENTRY(end_item), mstimstr(t) );
	set_stop(end_item, NULL);
	break;
    }

    a->this.anntyp = END_ANALYSIS;
    a->this.subtyp = a->this.num = 0;
    a->this.chan = 127;
    a->this.aux = NULL;
    a->this.time = t;
    insert_annotation(a);
    if (redraw || (display_start_time <= end_analysis_time &&
		   end_analysis_time < display_start_time + nsamp)) {
      clear_annotation_display();
      show_annotations(display_start_time, nsamp);
    }
  }
}

void set_back(GtkButton *button, gpointer user_data)
{
  long step = end_analysis_time - begin_analysis_time, t0, t1;
  
  if (begin_analysis_time <= 0L || step <= 0L) return;
  if ((t0 = begin_analysis_time - step) < 0L) t0 = 0L;
  t1 = t0 + step;
  gtk_entry_set_text( GTK_ENTRY(start_item), mstimstr(t0)); 
  set_start(start_item, NULL);
  gtk_entry_set_text( GTK_ENTRY(end_item), mstimstr(t1)); 
  set_stop(end_item, NULL);
}

void set_ahead(GtkButton *button, gpointer user_data)
{
  long step = end_analysis_time - begin_analysis_time, t0, t1,
    te = strtim("e");

  if ((te > 0L && end_analysis_time >= te) || step <= 0L) return;
  t0 = begin_analysis_time + step;
  t1 = t0 + step;
  gtk_entry_set_text( GTK_ENTRY(end_item), mstimstr(t1)); 
  set_stop(end_item, NULL);
  gtk_entry_set_text( GTK_ENTRY(start_item), mstimstr(t1)); 
  set_start(start_item, NULL);
}


void reset_start()
{
  if (analyze_popup_active >= 0) {
    char *p;
    
    if (begin_analysis_time == -1L) begin_analysis_time = 0L;
    gtk_entry_set_text( GTK_ENTRY(start_item), begin_analysis_time == 0L ?
			"beginning" : mstimstr(begin_analysis_time) );
    p = mstimstr(-begin_analysis_time);
    if (*p == '[') {
      *(p+13) = *(p+24) =  '\0';
      gtk_entry_set_text( GTK_ENTRY(astart_item), p+1);
      gtk_entry_set_text( GTK_ENTRY(dstart_item), p+14);
    }
    else {
      gtk_editable_delete_text( GTK_EDITABLE(astart_item), 0, -1);
      gtk_editable_delete_text( GTK_EDITABLE(dstart_item), 0, -1);
    }
  }
}

void reset_stop()
{
  if (analyze_popup_active >= 0) {
    char *p;

    if (end_analysis_time == -1L) end_analysis_time = strtim("e");
    gtk_entry_set_text( GTK_ENTRY(end_item), end_analysis_time == 0L ? 
			"end" : mstimstr(end_analysis_time));
    p = mstimstr(-end_analysis_time);
    if (*p == '[') {
      *(p+13) = *(p+24) =  '\0';
      gtk_entry_set_text( GTK_ENTRY(aend_item), p+1);
      gtk_entry_set_text( GTK_ENTRY(dend_item), p+14);
    }
    else {
      gtk_editable_delete_text( GTK_EDITABLE(aend_item), 0, -1);
      gtk_editable_delete_text( GTK_EDITABLE(dend_item), 0, -1);
    }
  }
}

void reset_siglist()
{
  if (analyze_popup_active >= 0) {
    char *p, sigliststring[WFDB_MAXSIG*4];
    int i;

    p = sigliststring;
    *p = '\0';
    for (i = 0; i < siglistlen; i++) {
      sprintf(p, "%d ", siglist[i]);
      p += strlen(p);
    }
    gtk_entry_set_text( GTK_ENTRY(siglist_item), sigliststring);
  }
  if (sig_mode)
    set_baselines();
}


void reset_maxsig()
{
  if (analyze_popup_active >= 0) { 
    if (signal_choice >= nsig || signal_choice < 0)
      signal_choice = 0;
    gtk_spin_button_set_adjustment( GTK_SPIN_BUTTON(signal_item),
				    GTK_ADJUSTMENT(gtk_adjustment_new(signal_choice, 
				    0, nsig-1, 1, 1, 1) ));
    gtk_label_set_text( GTK_LABEL(signal_name_item), signame[signal_choice]);
  }
}

/* Signal-list manipulation. */
void add_to_siglist(i)
int i;
{
  if (siglistlen < WFDB_MAXSIG && 0 <= i && i < nsig) {
    siglist[siglistlen++] = i;
    reset_siglist();
  }
}

void delete_from_siglist(i)
int i;
{
  int nsl;

  for (nsl = 0; nsl < siglistlen; nsl++) {
    if (siglist[nsl] == i) {
      siglistlen--;
      for ( ; nsl < siglistlen; nsl++)
	siglist[nsl] = siglist[nsl+1];
      reset_siglist();
    }
  }
}

void add_signal_choice()
{
  add_to_siglist(signal_choice);
}

void delete_signal_choice()
{
  delete_from_siglist(signal_choice);
}

/* This function executes the command string provided as its argument, after
   substituting for WAVE's internal variables (RECORD, ANNOTATOR, etc.). */

void do_command(p1)
char *p1;
{
  char *p2, *tp;
  
  post_changes();	/* make sure that the annotation file is up-to-date */
  finish_log();	/* make sure that the log file is up-to-date */
  if (analyze_popup_active < 0) create_analyze_popup();
  for (p2 = p1; *p2; p2++) {
    if (*p2 == '$') {
      console_input( console, p1, p2-p1 );
      p1 = ++p2;
      if (strncmp(p1, "RECORD", 6) == 0) {
	console_input( console, record, strlen(record));
	p1 = p2 += 6;
	    }
      else if (strncmp(p1, "ANNOTATOR", 9) == 0) {
	if (annotator[0][0])
	  console_input( console, annotator[0], strlen(annotator[0]));
	else
	  console_input( console, "\"\"", 2);
	p1 = p2 += 9;
      }
      else if (strncmp(p1, "START", 5) == 0) {
	if (begin_analysis_time != -1L && begin_analysis_time != 0L) {
	  tp = mstimstr(begin_analysis_time);
	  while (*tp == ' ') tp++;
	}
	else tp = "0";
	console_input( console, tp, strlen(tp));
	p1 = p2 += 5;
      }
      else if (strncmp(p1, "END", 3) == 0) {
	if (end_analysis_time != -1L)
	  tp = mstimstr(end_analysis_time);
	else if (end_analysis_time == 0L)
	  tp = "0";
	else tp = mstimstr(strtim("e"));
	while (*tp == ' ') tp++;
	console_input( console, tp, strlen(tp));
	p1 = p2 += 3;
      }
      else if (strncmp(p1, "DURATION", 8) == 0) {
	long t0 = begin_analysis_time, t1 = end_analysis_time;
	
	/* If end_analysis_time is unspecified, determine the
	   record length. */
	if (t1 == -1L) t1 = strtim("e");
	/* If the record length is also unspecified, use zero
	   in place of the duration.  Programs that accept
	   duration arguments must be written to accept zero
	   as meaning "unspecified". */
	if (t1 == 0L) console_input( console, "0", 1);
	else {
	  if (t0 == -1L) t0 = 0L;
	  tp = mstimstr(t1-t0);
	  while (*tp == ' ') tp++;
	  console_input( console, tp, strlen(tp));
	}
	p1 = p2 += 8;
      }		    
      else if (strncmp(p1, "SIGNALS", 7) == 0) {
	char s[4];
	int nsl;
	
	for (nsl = 0; nsl < siglistlen; nsl++) {
	  sprintf(s, "%d%s", siglist[nsl],
		  nsl < siglistlen ? " " : "");
	  console_input( console, s, strlen(s));
	}
	p1 = p2 += 7;
      }
      else if (strncmp(p1, "SIGNAL", 6) == 0) {
	char s[3];
	
	sprintf(s, "%d", signal_choice);
	console_input( console, s, strlen(s));
	p1 = p2 += 6;
      }
      else if (strncmp(p1, "LEFT", 4) == 0) {
	if (display_start_time < 1L) tp = "0";
	else {
	  tp = mstimstr(display_start_time);
	  while (*tp == ' ') tp++;
	}
	console_input( console, tp, strlen(tp));
	p1 = p2 += 4;
      }
      else if (strncmp(p1, "RIGHT", 5) == 0) {
	tp = mstimstr(display_start_time + nsamp);
	while (*tp == ' ') tp++;
	console_input( console, tp, strlen(tp));
	p1 = p2 += 5;
      }
      else if (strncmp(p1, "WIDTH", 5) == 0) {
	tp = mstimstr(nsamp);
	while (*tp == ' ') tp++;
	console_input( console, tp, strlen(tp));
	p1 = p2 += 5;
      }		    
      else if (strncmp(p1, "LOG", 3) == 0) {
	if (*log_file_name == '0')
	  sprintf(log_file_name, "log.%s", record);
	console_input( console, log_file_name, strlen(log_file_name));
	p1 = p2 += 3;
      }
      else if (strncmp(p1, "WFDBCAL", 7) == 0) {
	console_input( console, cfname, strlen(cfname));
	p1 = p2 += 7;
      }
      else if (strncmp(p1, "WFDB", 4) == 0) {
	console_input( console, getwfdb(), strlen(getwfdb()));
	p1 = p2 += 4;
      }
      else if (strncmp(p1, "TSCALE", 6) == 0) {
	char s[10];
	
	sprintf(s, "%g", mmpersec);
	console_input( console, s, strlen(s));
	p1 = p2 += 6;
      }
      else if (strncmp(p1, "VSCALE", 6) == 0) {
	char s[10];
	
	sprintf(s, "%g", mmpermv);
	console_input( console, s, strlen(s));
	p1 = p2 += 6;
      }
      else if (strncmp(p1, "DISPMODE", 8) == 0) {
	char s[3];
	
	sprintf(s, "%d", (ann_mode << 1) | show_marker);
	console_input( console, s, strlen(s));
	p1 = p2 += 8;
      }
      else if (strncmp(p1, "PSPRINT", 7) == 0) {
	console_input( console, psprint, strlen(psprint));
	p1 = p2 += 7;
      }
      else if (strncmp(p1, "TEXTPRINT", 7) == 0) {
	console_input( console, textprint, strlen(textprint));
	p1 = p2 += 9;
      }
      else if (strncmp(p1, "URL", 3) == 0) {
	console_input( console, url, strlen(url));
	p1 = p2 += 3;
      }
      else
	p1--;
    }
  }
  console_input( console, p1, p2-p1);
}

void do_analysis(GtkButton *button, gpointer user_data)
{ 
  do_command((char *)user_data);
}

static gchar tempfilename[20];

gboolean check_if_done()
{
  FILE *tfile;
  static int file_ready;
  void reinitialize();
  
  if (file_ready) {
    unlink(tempfilename);
    file_ready = 0;
    reinitialize(NULL,NULL);
    set_start(start_item, NULL);
    set_stop(end_item, NULL);
    return (FALSE);
  }
  if ((tfile = fopen(tempfilename, "r")) == NULL)
    return (TRUE);
  fclose(tfile);
  file_ready++;
  return (TRUE);
}

void
reload(GtkButton *button, gpointer user_data)
{
  static char command[80];

  sprintf(tempfilename, "/tmp/wave.XXXXXX");
  mkstemp(tempfilename);
  sprintf(command, "touch %s\n", tempfilename);
  
  do_command(command);
  
  gtk_timeout_add( 100, check_if_done, NULL);

}
