/* file: defaults.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions for reading and saving Wave's defalults

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "defaults.h"


GString *defaults_file_name;
GHashTable *db;

gint 
set_defaults_file( gchar *file ) {
  
  if ( ! file )
    return 0;

  if ( defaults_file_name )
    g_string_free( defaults_file_name, TRUE );
  
  defaults_file_name = g_string_new( file );

  return ( defaults_file_name->len > 0 );
}

gchar *
get_defaults_file( ) {
  return defaults_file_name->str;
}

// make this func return something sensable
gint
defaults_load_db( ) {

  FILE *INI;
  gint n;
  gchar buf[256], **entry=NULL, *key, *val;

  if ( ! (INI = fopen( defaults_file_name->str, "r")) ) 
    return -1;

  //g_printerr("info: opened %s for reading\n", defaults_file_name->str );

  if ( !db )
    db = g_hash_table_new(g_str_hash, g_str_equal);
  n = 0;
  while ( ! feof (INI) ) {
    fgets( buf, 256, INI);
    
    if( buf[0] != '#' ) {
      entry = g_strsplit( buf, ":", 1);
      if ( entry != NULL )
	if ( entry[0] != NULL ) 
	  if ( entry[1] != NULL ) {
	    key = g_strchug(entry[0]);
	    val = g_strchug(entry[1]);
	    defaults_set_string( key, val ); n++;
	  }
      g_strfreev( entry );
    }
  }
  fclose(INI);
  return n;
}


void 
write_to_file( gpointer key, gpointer value, gpointer file ) {
  fprintf( (FILE *)file, "%s: %s\n", (gchar *)key, (gchar *)value);
}

int
defaults_store_db( ) {
  FILE *INI;
  
  if ( ! (INI = fopen( defaults_file_name->str, "w")) ) {
    g_printerr("error: unable to open %s for writing\n", defaults_file_name->str );
    return -1;
  }
  
  g_hash_table_foreach(db, (GHFunc)write_to_file,INI);
  
  return fclose(INI);
}

void
defaults_set_string(gchar *key, gchar *val) {
  gchar *o_key=NULL, *o_val=NULL, *n_key=NULL, *n_val=NULL;

  if ( ! db ) {
    g_printerr("error: reference to unitialized default db");
    return;
  }
  n_key = g_malloc( strlen(key) + 1 );
  strcpy( n_key, key );
  n_val = g_malloc( strlen(val) + 1 );
  strcpy( n_val, val );  
  if ( g_hash_table_lookup_extended(db, &key, 
				    (gpointer *)&o_key, 
				    (gpointer *)&o_val) ) { 
    g_free(o_key);
    g_free(o_val);
  }
  g_hash_table_insert(db, n_key, n_val);
}

void
defaults_set_integer(gchar *key, gint val) {
  gchar str_val[32];
  g_snprintf( str_val, 32, "%d", val );
  defaults_set_string(key,str_val);
}

void
defaults_set_boolean(gchar *key, gboolean val) {
  gchar *str_val = val ? "1" : "0";
  defaults_set_string(key,str_val);
}

gchar *
defaults_get_string(gchar *key, gchar *default_val) {
  gchar *val;
  if ( ! db )
    return default_val;
  val = g_hash_table_lookup(db, key);
  if (val) g_strstrip(val);
  return  val ? val : default_val;
}

gint
defaults_get_integer(gchar *key, gint default_val) {
  gchar *val;
  if ( ! db )
    return default_val;
  val = g_hash_table_lookup(db, key);
  return val ? (int)strtol(val,NULL,10) : default_val;
}

gboolean
defaults_get_boolean(gchar *key, gboolean default_val) {
  char *val;
  if ( ! db )
    return default_val;
  val = g_hash_table_lookup(db, key);
  return val ? (gboolean)strtol(val,NULL,2) : default_val;
}

gboolean
hash_free(gpointer key, gpointer val, gpointer user_data) 
{
  g_free( (gchar *)key );
  g_free( (gchar *)val );
  return TRUE;
}

void
defaults_destroy_db() {
  g_hash_table_foreach_remove(db, hash_free, NULL);
  g_hash_table_destroy(db);
  g_string_free( defaults_file_name, TRUE );
}

