/* file: gtkwave.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions implementing Wave with the GTK+ toolkit

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#define INIT
#include "wave.h"
#include "gwave.h"
#include "defaults.h"
#include "interface.h"
#include "support.h"
#include <signal.h>		/* for SIGUSR1 definition */
#include <string.h>		/* for strchr definition */
#include <sys/unistd.h>
//#include <sys/stat.h>           /* for reading file status */
#ifdef __WIN32__
#else
#include <pwd.h>		/* for struct passwd definition */
#endif

#define NUM_C     6
#define BG_C      5

static int in_gtk_main_loop;
static int restore_all = 1;
static GdkColor color[NUM_C];
static GdkColor clear = {0, 0x0000, 0x0000, 0x0000};
static GdkColor set =   {1, 0xFFFF, 0xFFFF, 0xFFFF};

void initialize_layers();
GdkGC *get_pen(gint i);

/* Handle exposures in the signal window. */
gboolean on_canvas_expose_event(GtkWidget *widget,
				GdkEventExpose  *event,
				gpointer         user_data)
{
  gdk_draw_pixmap(widget->window,
		  widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
		  backend_pixmap,
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);

  return FALSE;
}

void
clear_all_layers() {
  gint i;
  
  for ( i=0; i<NUM_L; i++ ) 
    clear_layer(i);  
}

void
clear_layer(gint i) { 
  gdk_draw_rectangle( layer[i].bitmap, layer[i].clear, 
		      TRUE, 0, 0, canvas_width, canvas_height );
}


void
update_backend( ) {
  gint i; 
  GdkGC *pen;

  gdk_draw_rectangle (backend_pixmap, bg_fill, TRUE, 
		      0, 0, canvas_width, canvas_height);
  
  for (i= !visible; i<NUM_L; i++) {
    pen = get_pen(i);
    gdk_draw_rectangle( backend_pixmap, pen, TRUE, 0, 0,
  			canvas_width, canvas_height);
    gdk_gc_destroy(pen);
  }
}

void
update_screen( ) {
  GdkRectangle update_rect;

  update_backend();

  update_rect.x = 0;
  update_rect.y = 0;
  update_rect.width = canvas->allocation.width;
  update_rect.height = canvas->allocation.height;
  gtk_widget_draw(canvas, &update_rect);
}


gboolean
on_canvas_configure_event (GtkWidget       *widget,
			   GdkEventConfigure *event,
			   gpointer         user_data)
{  
  gint width = widget->allocation.width;
  gint height = widget->allocation.height;
  gint canvas_width_mm, u;


  if (backend_pixmap)
    gdk_pixmap_unref(backend_pixmap);
  backend_pixmap = gdk_pixmap_new(widget->window,
				  widget->allocation.width,
				  widget->allocation.height,
				  -1);
  bg_fill = gdk_gc_new( backend_pixmap );
  gdk_gc_set_foreground ( bg_fill, &color[BG_C] );

  initialize_layers(widget);

   /* The manipulations below are intended to select a canvas width that
     will allow a (reasonably) standard time interval to be displayed.
     If the canvas is at least 125 mm wide, its width is truncated to
     a multiple of 25 mm;  otherwise, its width is truncated to a multiple
     of 5 mm. */
  canvas_width_mm = width/dmmx(1);
  if (canvas_width_mm > 125) {
    canvas_width_sec = canvas_width_mm / 25;
    canvas_width_mm = 25 * canvas_width_sec;
  }
  else {
    u = canvas_width_mm / 5;
    canvas_width_mm = u * 5;
    canvas_width_sec = u * 0.2;
  }
  canvas_width = mmx(canvas_width_mm);
  if (width != canvas_width) {
    gdk_draw_rectangle(backend_pixmap, bg_fill, 
		       TRUE, canvas_width, 0,
		       width - canvas_width, height);
    width = canvas_width;
  }
  
  /* Similar code might be used to readjust the canvas height, but doing so
     seems unnecessary. */
  canvas_height = height;
  canvas_height_mv = canvas_height / dmmy(10);
 
  /* Recalibrate based on selected scales, clear the display list cache. */
  if (*record) {
    set_baselines();
    set_modes();	/* read and set user-selected scales, if any */
    vscale[0] = 0.;	/* force clear_cache() -- see calibrate() */
    calibrate();
  }
  
  restore_all = 1;	/* be sure that the background is repainted */
  clear_all_layers();
  restore_grid();
  do_disp( );
  restore_cursor();
  update_backend();

  return TRUE;
}

gint
read_wave_defaults(pargc, argv)
int *pargc;
char *argv[];
{
  gint rval = -1;
  if ( set_defaults_file( get_wave_filename( WAVE_DEFAULTS_FILE | WAVE_FILE_ENV ) ) )
    rval = defaults_load_db();

  if ( rval > 0 )
    return rval;
  
  if ( set_defaults_file( get_wave_filename( WAVE_DEFAULTS_FILE | WAVE_FILE_GLOBAL ) ) )
    rval = defaults_load_db(); 
  
  if ( set_defaults_file( get_wave_filename( WAVE_DEFAULTS_FILE | WAVE_FILE_USER ) ) )
    return ( defaults_load_db() );

  return rval;
}

void set_wave_dirs( ) {

  /* Get HOME environment variable  */
  homedir = g_string_new( g_getenv("HOME") == NULL ? "" : g_getenv("HOME") ); 
  if ( homedir->len < 1 ) 
    g_string_assign( homedir, HOMEDIR == NULL ? "" : HOMEDIR );
  if ( homedir->len > 1 ) {
    if ( homedir->str[ homedir->len-1 ] != G_DIR_SEPARATOR )
      g_string_append_c(homedir, G_DIR_SEPARATOR);
#ifndef __WIN32__
    g_string_append(homedir, ".wave");
    g_string_append_c(homedir, G_DIR_SEPARATOR);
#endif
  }
  else 
    g_string_free( homedir, TRUE );
  
  /* Get WAVEROOT environment variable  */
  wavedir = g_string_new( g_getenv("WAVEROOT") == NULL ? "" : g_getenv("WAVEROOT") );
  if ( wavedir->len < 1 ) 
    g_string_assign( wavedir, WAVEROOT );
  if ( wavedir->str[ wavedir->len-1 ] != G_DIR_SEPARATOR )
    g_string_append_c(wavedir, G_DIR_SEPARATOR);
 
  helpdir = g_string_new( HELPDIR );

}

gchar *
get_wave_filename( gchar type ) {
  gchar wave_file_type;
  gchar *tmp=NULL, *rval = NULL;
  GString *path = NULL;

  wave_file_type = type & 0x07;

  /* get global file */
  if ( (type & WAVE_FILE_GLOBAL) && ( wavedir->len > 0 ) ) {
    path = g_string_new( wavedir->str );
    
    switch ( wave_file_type ) {
    case WAVE_DEFAULTS_FILE:
      g_string_append( path, "Defaults" );
      break;
    case WAVE_ANNTAB_FILE:
      g_string_append( path, "anntab");
      break;
    case WAVE_MENU_FILE:
      g_string_append( path, "wavemenu.def");
      break;
    }
#ifdef __WIN32__
    rval = path->str;
    g_string_free( path, FALSE );
#else
    if ( access( path->str , R_OK ) == 0 ) {
      rval = path->str;
      g_string_free( path, FALSE );
    }
    else
      g_string_free( path, TRUE );
#endif
  }

  /* get user file */
  if ( ( type & WAVE_FILE_USER ) && ( homedir->len > 0 ) ) {
    path = g_string_new( homedir->str );
    
    switch ( wave_file_type ) {
    case WAVE_DEFAULTS_FILE:
      g_string_append( path, "Defaults" );
      break;
    case WAVE_ANNTAB_FILE:
      g_string_append( path, "anntab");
      break;
    case WAVE_MENU_FILE:
      g_string_append( path, "wavemenu.def");
      break;
    }
    if ( type & WAVE_FILE_EDITABLE ) {
      if ( access( path->str, W_OK ) ) {
	rval = path->str;
	g_string_free( path, FALSE );
      }
      else
	g_string_free( path, TRUE );
    }
#ifdef __WIN32__
    rval = path->str;
    g_string_free( path, FALSE );
#else
    else if ( access( path->str , R_OK ) == 0 ) {
      rval = path->str;
      g_string_free( path, FALSE );
    }
    else
      g_string_free( path, TRUE );
#endif
  }

  /* get by default ( in Defaults file ) */
  /* WAVE_DEFAULT_FILE cannot be set in Defaults */
  if ( type & WAVE_FILE_DEF ) {
    switch ( wave_file_type ) {
    case WAVE_ANNTAB_FILE:
      tmp = defaults_get_string( "Anntab", NULL );
      break;
    case WAVE_MENU_FILE:
      tmp = defaults_get_string( "Wavemenu", NULL );
      break;
    }
#ifdef __WIN32__
    rval = tmp;
#else
    if ( access( tmp , R_OK ) == 0 )
      rval = tmp;
#endif
  }

  /* set by environment variable */
  if ( type & WAVE_FILE_ENV ) {
    switch ( wave_file_type ) {
    case WAVE_DEFAULTS_FILE:
      tmp = g_getenv( "WAVEDEFAULTS" );
      break;
    case WAVE_ANNTAB_FILE:
      tmp = g_getenv( "ANNTAB" );
      break;
    case WAVE_MENU_FILE:
      tmp = g_getenv( "WAVEMENU" );
      break;
    }
#ifdef __WIN32__
    rval = tmp;
#else
    if ( access( tmp , R_OK ) == 0 )
      rval = tmp;
    else
      free(tmp);
#endif
  }

  //g_printerr("info: get_wave_file returning: %s\n", rval );
  return rval;
}

static int allowdottedlines;

/* Save current settings as new defaults. */
void save_defaults()
{
  gint result;
  char tstring[256];
  
  sprintf(tstring, "%gx%g", 25.4*dpmmx, 25.4*dpmmy);
  defaults_set_boolean("AllowDottedLines", allowdottedlines);
  defaults_set_string("Dpi", tstring);
  defaults_set_integer("SignalWindow.Height_mm",
		       (int)(canvas_height/(dpmmy*10))*10 + 10);
  defaults_set_integer("SignalWindow.Width_mm",
		       (int)(canvas_width/(dpmmx*25))*25 + 25);
  //defaults_set_integer("GraphicsMode", graphicsmode);
  defaults_set_boolean("View.Subtype", show_subtype);
  defaults_set_boolean("View.Chan", show_chan);
  defaults_set_boolean("View.Num", show_num);
  defaults_set_boolean("View.Aux", show_aux);
  defaults_set_boolean("View.Markers", show_marker);
  defaults_set_boolean("View.SignalNames", show_signame);
  defaults_set_boolean("View.Baselines", show_baseline);
  defaults_set_boolean("View.Level", show_level);
  if (sampfreq(NULL) >= 10.0)
    defaults_set_integer("View.TimeScale", tsa_index);
  else
    defaults_set_integer("View.CoarseTimeScale", tsa_index);
  defaults_set_integer("View.AmplitudeScale", vsa_index);
  defaults_set_integer("View.AnnotationMode", ann_mode);
  //defaults_set_integer("View.AnnotationOverlap", overlap);
  defaults_set_integer("View.SignalMode", sig_mode);
  defaults_set_integer("View.TimeMode", time_mode);
  if (tsa_index > MAX_COARSE_TSA_INDEX)
    defaults_set_integer("View.GridMode", grid_mode);
  else
    defaults_set_integer("View.CoarseGridMode", grid_mode);
  
  if ( access( get_defaults_file(), W_OK ) < 0 ) {
    result = notice_prompt("You are now using a read only default file, \
which you may not modify. Press `OK' to save the defaults to your
home directory instead",
			   "OK", "Cancel");
    
    if (result != NOTICE_YES)
      return;
    
    set_defaults_file( get_wave_filename( WAVE_DEFAULTS_FILE | WAVE_FILE_USER | WAVE_FILE_EDITABLE ) );    
  }

  defaults_store_db();
}

static char sentinel[30];

void
destroy_func                           (GtkObject       *object,
                                        gpointer         user_data)
{
  if ( post_changes() )
    finish_log();
#ifdef __WIN32__
#else
  unlink(sentinel);
#endif
  gtk_main_quit();
}

/*
initialize_graphics() 
Set the variables used within the gtk callbacks generated by glade, then
call glade-provided initialization functions to create the wave interface.
*/
int initialize_graphics(mode)
int mode;
{
  
  char *annfontname, *cname, *rstring;
  int height_mm, width_mm;		/* screen dimensions, in millimeters */
  int height_px, width_px;		/* screen dimensions, in pixels */
  int hmmpref = 120, wmmpref = 250;	/* preferred height and width of signal
					   window, in millimeters */

  /* print out all wave files for debuging */
  /*
  FILE * debug = fopen("debug.txt","w"); 
  fprintf(debug, "info: program name = %s\n", pname->str );
  fprintf(debug, "info: wavedir = %s\n", wavedir->str );
  fprintf(debug, "info: homedir = %s\n", homedir->str );
  fprintf(debug, "info: defaults = %s\n", 
	  get_wave_filename( WAVE_DEFAULTS_FILE | WAVE_FILE_GLOBAL ));
  fprintf(debug, "info: anntab = %s\n", 
	  get_wave_filename( WAVE_ANNTAB_FILE | WAVE_FILE_GLOBAL ));
  fprintf(debug, "info: menu = %s\n", 
	  get_wave_filename( WAVE_MENU_FILE | WAVE_FILE_GLOBAL ));
  switch ( get_os() ) {
  case OS_WIN95:
    fprintf(debug, "info: os is Windows 95\n");
    break;
  case OS_WINNT:
    fprintf(debug, "info: os is Windows NT\n");
    break;
  default:
    fprintf(debug, "info: os is not Windows 95 or NT\n");
  }
  fclose (debug);
  */

  /* this pixmap stuff was added by glade not sure if it is
     needed yet for now keep with initial_graphics */
/*add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps"); */
/*add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps"); */

  /* this was instered by glade not sure what it does yet */
  /*
#ifdef ENABLE_NLS
  bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
  textdomain (PACKAGE);
#endif
  */

  /* call glade generated function to create gtk interface,
     then get pointers to all widgets that will be needed */

  /* Get information about the dimensions, resolution, and type of the
     display available to this program. */
  
  if (dpmmx == 0) {
    rstring = defaults_get_string("Dpi", "100x100");
    (void)sscanf(rstring, "%lfx%lf", &dpmmx, &dpmmy);
    dpmmx /= 25.4;	/* convert dots/inch into dots/mm */
    dpmmy /= 25.4;
  }

  height_mm = gdk_screen_height_mm();
  height_px = gdk_screen_height();
  if (height_mm > 0) {
    if (dpmmy == 0.) dpmmy = ((double)height_px)/height_mm;
    else height_mm = height_px/dpmmy;
  }
  else { dpmmy = DPMMY; height_mm = height_px/dpmmy; }
  width_mm = gdk_screen_width_mm();
  width_px = gdk_screen_width();
  if (width_mm > 0) {
    if (dpmmx == 0.) dpmmx = ((double)width_px)/width_mm;
    else width_mm = width_px/dpmmx;
  }
  else { dpmmx = DPMMX; width_mm = width_px/dpmmx; }

  /* Choose default sizes for the frame and the signal window. */
  if (width_mm < 53 || height_mm < 75) {
    g_printerr("%s: display too small\n", pname->str);
    return (1);
  }

  wmmpref = defaults_get_integer("SignalWindow.Width_mm", 251);
  if (wmmpref > width_mm - 3) wmmpref = width_mm - 3;
  else if (wmmpref < 50) wmmpref = 50;

  hmmpref = defaults_get_integer("SignalWindow.Height_mm", 120);
  if (hmmpref > height_mm - 25) hmmpref = height_mm - 25;
  else if (hmmpref < 50) hmmpref = 50;
  canvas_width = mmx(wmmpref);
  canvas_height = mmy(hmmpref);
  linesp = mmy(4);
  
  /* Create frame for the main window. */
  gtk_wave = create_wave();
  canvas = lookup_widget( GTK_WIDGET(gtk_wave), "canvas");
  footer.right = lookup_widget(GTK_WIDGET(gtk_wave), "footer_right");
  footer.left = lookup_widget(GTK_WIDGET(gtk_wave), "footer_left");

  /* Create the sentinel file. */
#ifdef __WIN32__
#else
  sprintf(sentinel, "/tmp/.wave.%d.%d", (int)getuid(), (int)getpid());
  fclose(fopen(sentinel, "w"));	/* create as an empty file */
#endif
  /* Create the `View' panel, read the resource database, and set the `View'
     options accordingly. */
  create_mode_popup();

  if (!show_subtype)
    show_subtype = defaults_get_boolean("View.Subtype",0);
  if (!show_chan)
    show_chan = defaults_get_boolean("View.Chan",0);
  if (!show_num)
    show_num = defaults_get_boolean("View.Num",0);
  if (!show_aux)
    show_aux = defaults_get_boolean("View.Aux",0);
  if (!show_marker)
    show_marker = defaults_get_boolean("View.Markers",0);
  if (!show_signame)
    show_signame = defaults_get_boolean("View.SignalNames",0);
  if (!show_baseline)
    show_baseline = defaults_get_boolean("View.Baselines",0);
  if (!show_level)
    show_level = defaults_get_boolean("View.Level",0);

  if (tsa_index < 0) {
    tsa_index = fine_tsa_index = 
      defaults_get_integer("View.TimeScale",DEF_TSA_INDEX);
    coarse_tsa_index = 
      defaults_get_integer("View.CoarseTimeScale",DEF_COARSE_TSA_INDEX);
  }
  if (vsa_index < 0)
    vsa_index = defaults_get_integer("View.AmplitudeScale",
				     DEF_VSA_INDEX);
  if (ann_mode < 0)
    ann_mode = defaults_get_integer("View.AnnotationMode",0);

  //if (overlap < 0)
  //  overlap = defaults_get_integer("View.AnnotationOverlap",0);

  if (sig_mode < 0)
    sig_mode = defaults_get_integer("View.SignalMode",0);

  if (time_mode < 0)
    time_mode = defaults_get_integer("View.TimeMode",0);

  if (grid_mode < 0) {
    grid_mode = fine_grid_mode = defaults_get_integer("View.GridMode",0);
    coarse_grid_mode = defaults_get_integer("View.CoarseGridMode",0);
  }
  mode_undo();
  
  create_main_panel();

  gtk_widget_set_usize(canvas, canvas_width, canvas_height);

  /* Set the desired RGB values. */
  cname = defaults_get_string("SignalWindow.BackgroundColor","white");
  gdk_color_parse(cname, &color[BG_C]);
  gdk_colormap_alloc_color(gdk_colormap_get_system (), &color[BG_C], FALSE, TRUE);

  cname = defaults_get_string("SignalWindow.GridColor","grey90");
  gdk_color_parse(cname, &color[GRID_L]);
  gdk_colormap_alloc_color(gdk_colormap_get_system (), &color[GRID_L], FALSE, TRUE);

  cname = defaults_get_string("SignalWindow.SignalColor","blue");
  gdk_color_parse(cname, &color[SIGNAL_L]);
  gdk_colormap_alloc_color(gdk_colormap_get_system (), &color[SIGNAL_L], FALSE, TRUE);

  cname = defaults_get_string("SignalWindow.AnnotationColor","yellow green");
  gdk_color_parse(cname, &color[ANNOT_L]);
  gdk_colormap_alloc_color(gdk_colormap_get_system (), &color[ANNOT_L], FALSE, TRUE);

  cname = defaults_get_string("SignalWindow.CursorColor","orange red");
  gdk_color_parse(cname, &color[CURSOR_L]);
  gdk_colormap_alloc_color(gdk_colormap_get_system (), &color[CURSOR_L], FALSE, TRUE);
 
  color[HSIGNAL_L] = color[CURSOR_L];

  /* -- get the font -- */
  annfontname = defaults_get_string("SignalWindow.Font",DEFANNFONT);
  if ((font = gdk_font_load(annfontname)) == NULL) {
    g_printerr("can't find font %s", annfontname);
    g_printerr(" -- substituting font %s\n", DEFANNFONT);
  }

  /* get singal window cursor */
  cursor = gdk_cursor_new( GDK_CROSSHAIR );

  allowdottedlines = defaults_get_boolean("AllowDottedLines", 1);

  return 0;

}

GdkGC *get_pen( gint i ) {
  GdkGC *gc;

  gc = gdk_gc_new( backend_pixmap );
  gdk_gc_set_clip_origin( gc, 0, 0 );
  gdk_gc_set_clip_mask( gc, layer[i].bitmap );
  gdk_gc_set_foreground( gc, &color[i] );
  
  return gc;
}

void
initialize_layers(GtkWidget *widget) {
  gint i;
  static GdkGCValues draw_gcvalues;

  for ( i=0; i<NUM_L; i++ ) {
    /* -- get layer bitmap -- */
    if ( layer[i].bitmap )
      gdk_bitmap_unref( layer[i].bitmap );
    layer[i].bitmap = gdk_pixmap_new(NULL, widget->allocation.width, widget->allocation.height, 1);
  
    /* -- GC for drawing/clearing bitmap -- */
    if ( layer[i].draw )
      gdk_gc_unref( layer[i].draw );

    if ( layer[i].clear )
      gdk_gc_unref( layer[i].clear );
 
    draw_gcvalues.foreground = set;
    draw_gcvalues.font = font;
    draw_gcvalues.line_style = GDK_LINE_SOLID;

    switch ( i ) {

    /* -- setup Grid GC's -- */
    case GRID_L:
      /* -- GC for drawing bitmap -- */
      layer[i].draw = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					      GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE );
      /* -- GC for drawing bitmap -- */
      draw_gcvalues.foreground = clear;
      layer[i].clear = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					       GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE );
      
      /*-- GRID uses two sets of GC's for drawing/clearing bitmap -- */
      draw_gcvalues.foreground = set;
      draw_gcvalues.line_width = 2;
      layer[i].draw2 = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					       GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | 
					       GDK_GC_LINE_WIDTH);
      draw_gcvalues.foreground = clear;
      layer[i].clear2 = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
						GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | 
						GDK_GC_LINE_WIDTH);
      break;

    /* -- setup Signal GC's -- */
    case SIGNAL_L:
      layer[i].draw = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					      GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | GDK_GC_FONT );
      draw_gcvalues.foreground = clear;
      layer[i].clear = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					       GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | GDK_GC_FONT );
      break;

    /* -- setup Highlighted Signal GC's -- */
    case HSIGNAL_L:
      layer[i].draw = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					      GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | GDK_GC_FONT );
      draw_gcvalues.foreground = clear;
      layer[i].clear = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					       GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | GDK_GC_FONT );
      break;

    /* -- setup Annotation GC's -- */

    case ANNOT_L:
      /* -- GC for drawing bitmap -- */
      if (allowdottedlines)
	draw_gcvalues.line_style = GDK_LINE_ON_OFF_DASH;
      
      layer[i].draw = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					      GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | GDK_GC_FONT );
      draw_gcvalues.foreground = clear;
      layer[i].clear = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					       GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | GDK_GC_FONT );  
      
      break;
    
    /* -- setup Cursor GC's -- */
    case CURSOR_L:
      gdk_colormap_alloc_color (gdk_colormap_get_system (), &color[CURSOR_L], FALSE, TRUE);

      layer[i].draw = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					      GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | GDK_GC_FONT );
      draw_gcvalues.foreground = clear;
      layer[i].clear = gdk_gc_new_with_values( layer[i].bitmap, &draw_gcvalues,
					       GDK_GC_FOREGROUND | GDK_GC_LINE_STYLE | GDK_GC_FONT );
    }


  }
}

void display_and_process_events()
{ 

  gtk_widget_show (gtk_wave);
  gdk_window_set_cursor( canvas->window, cursor );
  
  /* search_template = create_search_template (); */
  /* gtk_widget_show (search_template); */

  in_gtk_main_loop = 1;
  gtk_main();

}

void
quit_proc (GtkButton       *button,
	   gpointer         user_data)
{
  if ( notice_prompt( "Are you sure you want to Quit?", "Confirm", "Cancel" ) 
       == NOTICE_YES ) gtk_widget_destroy(gtk_wave);   
}
