/* file: mainpan.c	I. Henry        20 July 2001
			Last revised:   20 July 2001
Functions for the main control panel of WAVE

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#include "wave.h"
#include "gwave.h"
#include "interface.h"
#include "support.h"
#include <wfdb/ecgcodes.h>
#include <sys/time.h>

#ifdef __WIN32__
int mkstemp(char *);
#endif

GtkWidget *load_frame;
GtkWidget *print_setup_frame;

GtkWidget *desc_item,	        /* Description field (from log file) */
          *annot_item,	        /* "Annotator:" field                */
          *find_item,	        /* "Search for:" field               */
	  *record_item,	        /* "Record:" field                   */
	  *time_item,	        /* "Start time:" field               */
    	  *time2_item,	        /* "End time:" field                 */
	  *wfdbpath_item,	/* "WFDB path:" field                */
	  *wfdbcal_item,	/* "Calibration file:" field         */
	  *psprint_item,	/* "PostScript print command:" field */
	  *textprint_item,	/* "Text print command:" field       */
          *show_subframe;

char wfdbpath[512];	/* database path */
char wfdbcal[128];	/* WFDB calibration file name */

void wfdbp_proc(GtkEditable *editable, gpointer user_data)
{
  char *p = gtk_entry_get_text( GTK_ENTRY (wfdbpath_item) );
  
  if (p != wfdbpath)
    strncpy(wfdbpath, p, sizeof(wfdbpath)-1);
  setwfdb(wfdbpath);
}

void wfdbc_proc(GtkEditable *editable, gpointer user_data)
{
  char *p = gtk_entry_get_text( GTK_ENTRY (wfdbcal_item) );

  if (strcmp(wfdbcal, p)) {
    strncpy(wfdbcal, p, sizeof(wfdbcal)-1);
    if (calopen(cfname = wfdbcal) == 0)
      calibrate();
  }
}

static void create_load_panel()
{

  char *p;
  void reinitialize();

  strncpy(wfdbpath, getwfdb(), sizeof(wfdbpath)-1);
  if (cfname != wfdbcal) {
    if (cfname != NULL)
      strncpy(wfdbcal, cfname, sizeof(wfdbcal)-1);
    else if ( (p = g_getenv("WFDBCAL")) )
      strncpy(wfdbcal, p, sizeof(wfdbcal)-1);
  }

  load_frame = create_file_load();
  
  record_item = lookup_widget(GTK_WIDGET( load_frame ), "record_entry");
  gtk_entry_set_text( GTK_ENTRY( record_item ), record );
  annot_item = lookup_widget(GTK_WIDGET( load_frame ), "annotator_entry");
  gtk_entry_set_text( GTK_ENTRY( annot_item ), annotator[0] );
  wfdbcal_item = lookup_widget(GTK_WIDGET( load_frame ), "calibration_entry");
  gtk_entry_set_text( GTK_ENTRY( wfdbcal_item ), wfdbcal );
  wfdbpath_item = lookup_widget(GTK_WIDGET( load_frame ), "path_entry");
  gtk_entry_set_text( GTK_ENTRY( wfdbpath_item ), wfdbpath );
  
}

void show_load(GtkMenuItem *menuitem, gpointer user_data)
{
  gtk_widget_show( GTK_WIDGET(load_frame) );
}

void
hide_load                              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide( GTK_WIDGET(load_frame) );
}


void print_setup_proc(GtkEditable *editable, gpointer user_data)
{
  strncpy(psprint, (char *)gtk_entry_get_text( GTK_ENTRY( psprint_item) ),
	  sizeof(psprint)-1);
  strncpy(textprint, (char *)gtk_entry_get_text( GTK_ENTRY (textprint_item) ),
	  sizeof(textprint)-1);
}

static void create_print_setup_panel()
{
  char *p, *printer;

  if ((p = g_getenv("TEXTPRINT")) == NULL || strlen(p) > sizeof(textprint)-1) {
    if ((printer = g_getenv("PRINTER")) == NULL)
      sprintf(textprint, "lpr");
    else
      sprintf(textprint, "lpr -P%s", printer);
  }
  else
    strcpy(textprint, p);
  if ((p = g_getenv("PSPRINT")) == NULL || strlen(p) > sizeof(psprint)-1) {
    if ((printer = g_getenv("PRINTER")) == NULL)
      sprintf(psprint, "lpr");
    else
      sprintf(psprint, "lpr -P%s", printer);
  }
  else
    strcpy(psprint, p);

  print_setup_frame = create_print_setup();

  psprint_item = lookup_widget(GTK_WIDGET( print_setup_frame ), "print_setup_postscript_entry");
  textprint_item = lookup_widget(GTK_WIDGET( print_setup_frame ), "print_setup_text_entry");
  gtk_entry_set_text( GTK_ENTRY( psprint_item ), psprint );
  gtk_entry_set_text( GTK_ENTRY( textprint_item ), textprint );
}

void show_print_setup(GtkMenuItem *menuitem, gpointer user_data)
{
  gtk_widget_show( GTK_WIDGET(print_setup_frame) );
}

void
hide_print_setup                       (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide( GTK_WIDGET(print_setup_frame) );
}


gboolean
on_print_setup_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  hide_print_setup(NULL,NULL);
  return TRUE;
}

static void create_file_menu()
{
  create_load_panel();
  create_print_setup_panel();
}

void allow_editing(GtkMenuItem *menuitem, gpointer user_data)
{
  accept_edit = 1;  
}

void view_only(GtkMenuItem *menuitem, gpointer user_data)
{
  accept_edit = 0;
}

static char filename[20], *title;

void
show_print(GtkButton *button, gpointer user_data)
{
    char print_command[128];

    if (filename) {
	if (strncmp(filename, "/tmp/wave-s", 11) == 0)
	    sprintf(print_command, "wfdbdesc $RECORD | %s\n", textprint);
	else if (strncmp(filename, "/tmp/wave-a", 11) == 0)
	    sprintf(print_command, "sumann -r $RECORD -a $ANNOTATOR | %s\n",
		    textprint);
	else
	    sprintf(print_command, "%s <%s\n", textprint, filename);
	do_command(print_command);
    }
}



static void show_file()
{
  GtkWidget *textsw;
  char buffer[255];
  gint status;
  FILE *fid;

  show_subframe = create_file_viewer();
  gtk_window_set_title( GTK_WINDOW(show_subframe), title ); 

  textsw = lookup_widget( GTK_WIDGET( show_subframe ), "file_viewer_text");

  status = 0;
  if ( (fid = fopen( filename, "r")) == NULL  ) {
    status = 1;
    g_printerr("unable to open %s\n", filename);
  }
  else {
    while ( !feof(fid) ) {
      fgets( buffer, 255, fid);
      status = 1;
      gtk_text_insert( GTK_TEXT(textsw), textsw->style->font, 
		       &(textsw->style->black), &(textsw->style->white), 
		       buffer, strlen(buffer) );
    }
  }

  if (!status) {
    notice_prompt("Sorry, no property information\nis available for this topic.", 
		  "Continue", NULL);  
    gtk_widget_destroy( GTK_WIDGET( show_subframe ) );
  }
  else 
    gtk_widget_show( GTK_WIDGET(show_subframe) );
  
}

/* This function gets called 10 times per second while the timer is running.
   Once the file named by filename contains readable data, it waits one
   more second, turns off the timer, invokes show_file, and then deletes
   the file. */
gboolean check_file()
{
  char tstring[20];
  FILE *tfile;
  static int file_ready;
  
  if (file_ready) {
    show_file();
    unlink(filename);
    file_ready = 0;
    return (FALSE);
  }
  if ((tfile = fopen(filename, "r")) == NULL)
    return (TRUE);
  if (fgets(tstring, 20, tfile) == NULL) {
    fclose(tfile);
    return (TRUE);
  }
  fclose(tfile);
  file_ready++;
  return (TRUE);
}

/* This function sets up the timer for check_file. */
void wait_for_file()
{
  gtk_timeout_add( 100, check_file, NULL);
}

static char command[80];

void prop_signals (GtkMenuItem     *menuitem,
		   gpointer         user_data)

{
    sprintf(filename, "/tmp/wave-s.XXXXXX");
    mkstemp(filename);
    /* The `echo' is to make sure that something gets written to the file,
       even if `wfdbdesc' doesn't work (so that check_file doesn't wait
       forever). */
    sprintf(command, "(wfdbdesc $RECORD; echo =====) >%s\n", filename);
    do_command(command);
    title = "Signals";
    wait_for_file( );
}

void prop_annotations(GtkMenuItem *menuitem, gpointer user_data)
{
    post_changes();
    sprintf(filename, "/tmp/wave-a.XXXXXX");
    mkstemp(filename);
    sprintf(command, "(sumann -r $RECORD -a $ANNOTATOR; echo =====) >%s\n", filename);
    do_command(command);
    title = "Annotations";
    wait_for_file( );
}

void prop_wave(GtkMenuItem     *menuitem,
	       gpointer         user_data)

{
  sprintf(filename, "%swave.pro", helpdir->str);
  title = "About WAVE";
  show_file();
}

static int reload_signals, reload_annotations;
void
reinitialize (GtkButton       *button,
	      gpointer         user_data)
{
  reload_annotations = reload_signals = 1;
  wfdbp_proc(NULL,NULL);
  wfdbc_proc(NULL,NULL);
  disp_proc(NULL, GINT_TO_POINTER((gint)'.'));
}

GtkWidget *find_frame;

static void create_find_panel()
{
  find_frame = create_find();

  time_item = lookup_widget( GTK_WIDGET(find_frame), "start_time_entry");
  time2_item = lookup_widget( GTK_WIDGET(find_frame), "end_time_entry");
  find_item = lookup_widget( GTK_WIDGET(find_frame), "search_for_entry");
}


void
show_find(GtkButton *button,
	  gpointer user_data)
{
  gtk_widget_show(find_frame);
}

void
hide_find                              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide( GTK_WIDGET(find_frame) );
}

gboolean
on_find_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  /* don't allow find to be deleted, hide instead */
  hide_find(NULL,NULL);
  return TRUE;
}


void create_main_panel()
{
  
  create_file_menu();
  //create_edit_menu();
  //create_prop_menu();
  create_find_panel();
  
  //if (make_sync_button)
  //  xv_create(panel, PANEL_BUTTON,
  //	      XV_HELP_DATA, "wave:sync",
  //	      PANEL_LABEL_STRING, "Sync",
  //	      PANEL_NOTIFY_PROC, sync_other_wave_processes,
  //	      0);
  
}

/* This function converts a noise mnemonic string into a noise subtype.  It
   returns -2 if the input string is not a noise mnemonic string. */
static int noise_strsub(s)
char *s;
{
  int i, imax, n = 0;
  
  if (('0' <= *s && *s <= '9') || strcmp(s, "-1") == 0)
    return (atoi(s));
  else if (strcmp(s, "U") == 0)
    return (-1);
  imax = (nsig <= 4) ? nsig : 4;
  if (strlen(s) != imax)
    return (-2);
  for (i = 0; i < imax; i++) {
    if (s[i] == 'c')
      continue;		/* signal i is clean */
    else if (s[i] == 'n')
      n |= (1 << i);	/* signal i is noisy */
    else if (s[i] == 'u')
      n |= (0x11 << i);	/* signal i is unreadable */
    else
      return (-2);	/* the string is not a noise mnemonic */
  }
  return (n);
}

void
disp_proc(GtkWidget *widget, gpointer user_data)
{
  int etype, i;
  long cache_time=0, next_match(), previous_match();
  void set_frame_footer();


  /* Reset display modes if necessary. */
  set_modes();

  /* If a new record has been selected, re-initialize. */
  if (reload_signals ||
      strncmp(record, (char *)gtk_entry_get_text(GTK_ENTRY(record_item)), RNLMAX)) {
    wfdbquit();
    
    /* Reclaim memory previously allocated for baseline labels, if any. */
    for (i = 0; i < nsig; i++)
      if (blabel[i]) {
	free(blabel[i]);
	blabel[i] = NULL;
      }
    
    if (!record_init((char *)gtk_entry_get_text(GTK_ENTRY(record_item)))) return;
    annotator[0][0] = '\0';	/* force re-initialization of annotator if
				   record was changed */
    savebackup = 1;
  }

  /* If a new annotator has been selected, re-initialize. */
  if (reload_annotations ||
      strncmp(annotator[0], (char *)gtk_entry_get_text(GTK_ENTRY(annot_item)),ANLMAX)) {
    strncpy(annotator[0], (char *)gtk_entry_get_text(GTK_ENTRY(annot_item)), ANLMAX);
    if (annotator[0][0]) {
      af[0].name = annotator[0]; af[0].stat = WFDB_READ;
      nann = 1;
    }
    else
      nann = 0;
    annot_init();
    savebackup = 1;
  }
  
  reload_signals = reload_annotations = 0;
  
  /* Get the event type.  This procedure is usually invoked with a Panel_item
     in item in response to a panel event, but it can also be called
     directly (with item = NULL and event = etype), as from window_event_proc
     in edit.c. */
  etype =  GPOINTER_TO_INT(user_data);

  /* Find out which button was pushed, and act on it. */
  switch (etype) {
  default:
  case '.':	/* Start at time specified on panel. */
  case '*':	/* (from scope_proc(), see scope.c */
  case '!':	/* (from show_next_entry(), see logpan.c */
    display_start_time = wstrtim((char *)gtk_entry_get_text(GTK_ENTRY(time_item)));
    if (display_start_time < 0L) display_start_time = -display_start_time;
    cache_time = -1L;
    break;
  case ':':	/* End at time specified on panel. */
    display_start_time = wstrtim((char *)gtk_entry_get_text( GTK_ENTRY(time2_item)));
    if (display_start_time < 0L) display_start_time = -display_start_time;
    if ((display_start_time -= nsamp) < 0L) display_start_time = 0L;
    cache_time = -1L;
    break;
  case 'h':	/* Go to the beginning of the record (see edit.c) */
    display_start_time = 0L;
    cache_time = -1L;
    break;
  case 'e':	/* Go to the end of the record (see edit.c) */
    if ((display_start_time = strtim("e") - nsamp) < 0L)
      display_start_time = 0L;
    cache_time = -1L;
    break;
  case ']':	/* Find next occurrence of specified annotation. */
  case '[':	/* Find previous occurrence of specified annotation. */
    if (annotator[0][0]) {
      char *fp = (char *)gtk_entry_get_text(GTK_ENTRY(find_item));
      static char auxstr[256];
      int mask, noise_mask, target;
      long t = 0;
      struct WFDB_ann template;
      
      if (*fp == '\0') {
	/* if find_item is empty, use the search template */
	template = search_template;
	mask = search_mask;
	if (template.aux == NULL) mask &= ~M_AUX;
      }
      else if ( (target = strann(fp)) ) {
	template.anntyp = target;
	mask = M_ANNTYP;
      }
      else if ((noise_mask = noise_strsub(fp)) >= -1) {
	template.anntyp = NOISE;
	template.subtyp = noise_mask;
	mask = M_ANNTYP | M_SUBTYP;
      }
      else if (strcmp(fp, ".") == 0) {
	template.anntyp = NOTQRS;
	mask = M_ANNTYP;
      }
      else if (strcmp(fp, ":") == 0) {
	template.anntyp = INDEX_MARK;
	mask = M_ANNTYP;
      }
      else if (strcmp(fp, "<") == 0) {
	template.anntyp = BEGIN_ANALYSIS;
	mask = M_ANNTYP;
      }
      else if (strcmp(fp, ">") == 0) {
	template.anntyp = END_ANALYSIS;
	mask = M_ANNTYP;
      }
      else if (strcmp(fp, "*n") == 0) {
	template.anntyp = NORMAL;
	mask = M_MAP2;
      }
      else if (strcmp(fp, "*s") == 0) {
	template.anntyp = SVPB;
	mask = M_MAP2;
      }
      else if (strcmp(fp, "*v") == 0) {
	template.anntyp = PVC;
	mask = M_MAP2;
      }
      else if (strcmp(fp, "*") == 0 || *fp == '\0')
      mask = 0;	/* match annotation of any type */
      else {
	strncpy(auxstr+1, fp, 255);
	auxstr[0] = strlen(auxstr+1);
	template.aux = auxstr;
	mask = M_AUX;
      }
      if ((etype == (int)']' && (t = next_match(&template, mask)) < 0L)||
	  (etype == (int)'[' && (t=previous_match(&template,mask)) <0L)){
	    notice_prompt("No match found!", "Continue", NULL);
	  }
      else {
	display_start_time = strtim(timstr(t-(long)((nsamp-freq)/2)));
	if (etype == (int)']') t = next_match(&template, mask);
	else t = previous_match(&template, mask);
	if (t > 0) cache_time=strtim(timstr(t-(long)((nsamp-freq)/2)));
	else cache_time = -1L;
      }
    }
    break;
  case '<':	/* Go backwards one frame. */
    if ((display_start_time -= nsamp) < 0) display_start_time = 0;
    cache_time = display_start_time - nsamp;
    break;
  case '(':	/* Go backwards one-half frame. */
    if ((display_start_time -= nsamp/2) < 0) display_start_time = 0;
    cache_time = display_start_time - nsamp/2;
    break;
  case ')':	/* Go forwards one-half frame. */
    display_start_time += nsamp/2;
    cache_time = display_start_time + nsamp/2;
    break;
  case '>':	/* Go forwards one frame. */
    display_start_time += nsamp;
    cache_time = display_start_time + nsamp;
    break;
  }
  
  if (etype != '!' && description[0]) {
    description[0] = '\0';
    set_window_title();
  }
  
  /* Erase the drawing region, and a little more.  The width of the canvas
     can be slightly greater than canvas_width (because of the adjustments
     performed by do_resize() in xvwave.h);  it is possible for text such
     as annotations to be written in this region to the right of the drawing
     area, and we want to be sure to erase it if this has happened. */
  bar(0,0,0);
  box(0,0,0);
  
  //gdk_draw_rectangle (backend_pixmap, bg_fill,
  //		      TRUE, 0, 0, canvas_width, canvas_height);
  //XFillRectangle(display, xid, clear_all,
  //		 0, 0, canvas_width+mmx(10), canvas_height);
  
  /* Reset the pointer for the scope display unless the scope is running or
     has just been paused. */
  if (!scan_active && etype != '*') {
    (void)locate_annotation(display_start_time, -128);
    scope_annp = annp;
  }
  
  /* Display the selected data. */

  do_disp();

  set_window_footer();
  
  /* Read ahead if possible. */
  if (cache_time >= 0) (void)find_display_list(cache_time);

  update_screen();

}

void set_record_item(char *s)
{ 
  gtk_entry_set_text( GTK_ENTRY( record_item ), s ); 
}

void set_annot_item(char *s)
{
  gtk_entry_set_text( GTK_ENTRY( annot_item ), s ); 
}

void set_start_time(char *s)
{
  gtk_entry_set_text( GTK_ENTRY( time_item ), s ); 
}

void set_end_time(char *s)
{
  gtk_entry_set_text( GTK_ENTRY( time2_item ), s ); 
}

void set_find_item(char *s)
{
  gtk_entry_set_text( GTK_ENTRY( find_item ), s ); 
}

gboolean on_file_load_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  hide_load(NULL,NULL);
  return TRUE;
}

void save_proc(GtkMenuItem *menuitem, gpointer user_data)
{
  if (post_changes())
    set_window_title();
}

void on_log_menu_activate(GtkMenuItem *menuitem, gpointer user_data)
{
  GtkWidget *log;
  log = create_log ();
  gtk_widget_show (log);
}

void on_help_button_clicked(GtkButton *button, gpointer user_data)
{

}


gboolean
on_file_viewer_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  dismiss_file_viewer(NULL,NULL);
  return FALSE;
}


void
dismiss_file_viewer                    (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy( show_subframe );
}


void
on_menu_quit_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  quit_proc(NULL, NULL);
}
