/* file: wave.c	        I. Henry        20 July 2001
			Last revised:   20 July 2001
The main loop function of WAVE

-------------------------------------------------------------------------------
GTKWave: Waveform analyzer, viewer, and editor (GTK+ port)
Copyright (C) 2001 George B. Moody and Isaac C. Henry

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA.

You may contact the author by e-mail (ihenry@physionet.org) or postal mail
(MIT Room E25-505A, Cambridge, MA 02139 USA).  For updates to this software,
please visit PhysioNet (http://www.physionet.org/).
_______________________________________________________________________________

*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "wave.h"
#include <gtk/gtk.h>
#include <string.h>		/* for strchr definition */


int
main (int argc, char *argv[])
{
  GString *wfdbp;
  gchar *p, *start_string = NULL, *tp;
  int do_demo = 0, i, mode = 0, win_mode = 0;
  static int wave_procno;
  
  /* Initialize non-zero global variables. */
  begin_analysis_time = end_analysis_time = -1L;
  tsa_index = vsa_index = ann_mode = overlap = sig_mode = time_mode =
    grid_mode = -1;
  tscale = 1.0;

  gtk_set_locale ();
  gtk_init(&argc, &argv);

  /* Extract program name for use in error messages. */
  pname = g_string_new( ( g_get_prgname() == NULL ) ? "wave" : g_get_prgname() );

  set_wave_dirs();

  /* Find the record name argument. */
  if ( argc == 2 ) 
    if (strcmp(argv[1], "-w") == 0)
      win_mode = 1;

  for (i = 1; i < argc-1; i++)
    if (strcmp(argv[i], "-r") == 0) 
      break;
  
  /* Provide (non-window based) usage information if needed. */
#ifdef __WIN32__
  if ( (i == argc-1 || argc == 1) && (win_mode == 0) ) 
    win_mode = 1; /* force window to open */
#else
  if ( (i == argc-1 || argc == 1 || g_getenv("DISPLAY") == NULL) && (win_mode == 0) ) 
    help();	/* print info and quit */
#endif


  /* Check for requests to open more than one record. */
  if ( win_mode == 0 ) {
    strcpy(record, argv[++i]);
    while ( (p = strchr(record, '+'))) {
      static int wave_pid;
      
      wave_pid = (int)getpid();	/* save the PID of this process */
      
#ifdef __WIN32__
#else
      if (fork() == 0) {		/* child */
	wave_ppid = wave_pid;
	wave_procno++;
	p++; tp = record;
	while ( (*tp++ = *p++) )
	  ;	/* delete the first record name from the string */
	strcpy(argv[i], record);
	if (strchr(record, '+') == NULL)
	  make_sync_button = 1;
      }
      else {	/* parent */
	*p = '\0';	/* delete everything after the first record from the
			   string */
	strcpy(argv[i], record);
      }
#endif
    }
  }
  /* Remove any command-line arguments that are specific to another WAVE
     process, and remove the `+n/' prefix from any that are specific to
     this process. */
  for (i = 1; i < argc; i++) {
    if (*argv[i] == '+') {
      if ((atoi(argv[i]+1) == wave_procno) && (p = strchr(argv[i], '/')))
	argv[i] = p+1;
      else {
	int j;
	for (j = i+1; j < argc; j++)
	  argv[j-1] = argv[j];
	argc--;
      }
    }
  }

  read_wave_defaults();

  /* Process WAVE-specific arguments. */
  for (i = 1; i < argc; i++) {
    if (*argv[i] == '-')
      switch (*(argv[i] + 1)) {
      case 'a':	/* annotator name */
	if (++i >= argc) {
	  g_printerr("%s: annotator name must follow -a\n",
		  pname->str);
	  exit(1);
	}
	if (strlen(argv[i]) > ANLMAX) {
	  g_printerr("%s: annotator name is too long\n",
		     pname->str);
	  exit(1);
	}
	strcpy(annotator[0], argv[i]);
	break;
      case 'd':	/* display resolution */
	if (strcmp(argv[i], "-dpi")) break;
	if (++i >= argc) {
	  g_printerr("%s: resolution must follow -dpi\n",
		     pname->str);
	  exit(1);
	}
	(void)sscanf(argv[i], "%lfx%lf", &dpmmx, &dpmmy);
	if (dpmmx < 0.) dpmmx = 0.;
	if (dpmmy <= 0.) dpmmy = dpmmx;
	dpmmx /= 25.4; dpmmy /= 25.4;
	break;
      case 'D': /* demo mode */
	if (++i >= argc) {
	  g_printerr("%s: log file name must follow -D\n",
		  pname->str);
	  exit(1);
	}
	if (strlen(argv[i]) > LNLMAX) {
	  g_printerr("%s: log file name is too long\n", pname->str);
	  exit(1);
	}
	strcpy(log_file_name, argv[i]);
	do_demo = 1;
	break;
      case 'f':	/* start at time specified in following argument */
	if (++i >= argc) {
	  g_printerr("%s: start time must follow -f\n",
		  pname->str);
	  exit(1);
	}
	start_string = argv[i];
	if (start_string[0] == '[')
	  time_mode = 1;	/* display absolute times */
	else if (start_string[0] == 's')
	  time_mode = 2;	/* display times in sample intervals */
	break;
      case 'g':	/* use shades of grey, even on color screen */
	mode |= MODE_GREY;
	break;
      case 'H':	/* use getvec's high-resolution mode for
		   multi-frequency records */
	setgvmode(WFDB_HIGHRES);
	break;
      case 'm':	/* run in monochrome, even on color screen */
	mode |= MODE_MONO;
	break;
      case 'p': /* add argument to WFDB path */
	if (++i >= argc) {
	  g_printerr("%s: input file location(s) must follow -p\n",
		     pname->str);
	  exit(1);
	}
	wfdb_addtopath(argv[i]);
	break;
      case 'r':	/* record name */
	if (++i >= argc) {
	  g_printerr("%s: record name must follow -r\n",
		  pname->str);
	  exit(1);
	}
	if (strlen(argv[i]) > RNLMAX) {
	  g_printerr("%s: record name is too long\n",
		  pname->str);
	  exit(1);
	}
	strcpy(record, argv[i]);
	break;
      case 's':	/* signal list follows */
	while (++i < argc && *argv[i] != '-') {
	  int sig;
	  
	  sig = atoi(argv[i]);
	  if (0 <= sig && sig < WFDB_MAXSIG &&
	      siglistlen < WFDB_MAXSIG){
	    siglist[siglistlen] = sig;
	    siglistlen++;
	  }
	}
	--i;
	if (siglistlen < 1) {
	  g_printerr("%s: one or more signal numbers must follow -s\n",
		     pname->str);
	  exit(1);
	}
	break;
      case 'S':   /* use shared color map only */
	mode |= MODE_SHARED;
	break;
      case 'V':	/* View options */
	switch (*(argv[i]+2)) {
	case 'a':	/* display annotation `aux' fields */
	  show_aux = 1;
	  break;
	case 'A':	/* set annotation display mode */
	  if (++i >= argc) {
	    g_printerr("%s: annotation display mode must follow -VA\n",
		       pname->str);
	    exit(1);
	  }
	  ann_mode = atoi(argv[i]);
	  break;
	case 'b':	/* display signal baselines */
	  show_baseline = 1;
	  break;
	case 'c':	/* display annotation `chan' fields */
	  show_chan = 1;
	  break;
	case 'G':	/* set grid display mode */
	  if (++i >= argc) {
	    g_printerr("%s: grid display mode must follow -VG\n",
		       pname->str);
	    exit(1);
	  }
	  grid_mode = coarse_grid_mode = fine_grid_mode =
	    atoi(argv[i]);
	  break;
	case 'l':	/* display cursor levels */
	  show_level = 1;
	  break;
	case 'm':	/* display annotation markers */
	  show_marker = 1;
	  break;
	case 'n':	/* display annotation `num' fields */
	  show_num = 1;
	  break;
	case 'N':	/* display signal names */
	  show_signame = 1;
	  break;
	case 's':	/* display annotation subtypes */
	  show_subtype = 1;
	  break;
	case 'S':	/* set signal display mode */
	  if (++i >= argc) {
	    g_printerr("%s: signal display mode must follow -VS\n",
		       pname->str);
	    exit(1);
	  }
	  sig_mode = atoi(argv[i]);
	  break;
	case 't':	/* set time scale array index */
	  if (++i >= argc) {
	    g_printerr("%s: time scale choice must follow -Vt\n",
		       pname->str);
	    exit(1);
	  }
	  tsa_index = coarse_tsa_index = fine_tsa_index =
	    atoi(argv[i]);
	  break;
	case 'T':	/* set time display mode */
	  if (++i >= argc) {
	    g_printerr("%s: time display mode must follow -VT\n",
		       pname->str);
	    exit(1);
	  }
	  time_mode = atoi(argv[i]);
	  break;
	case 'v':	/* set amplitude scale array index */
	  if (++i >= argc) {
	    g_printerr("%s: amplitude scale choice must follow -Vv\n",
		       pname->str);
	    exit(1);
	  }
	  vsa_index = atoi(argv[i]);
	  break;
	}
	break;
      }
  }
  
  /* Provide help if no record was specified. */
  if ( (record[0] == '\0') && (win_mode == 0)) help();

  /* Make sure that the WFDB path begins with an empty component (otherwise,
     annotation editing may become confused). */
  wfdbp = g_string_new( getwfdb() );
  if ( wfdbp->str[0] != PATH_SEP ) {
    g_string_prepend_c ( wfdbp, PATH_SEP  );
  }
  /* always make sure the wavedir is in the path */
  g_string_append_c( wfdbp, PATH_SEP );
  g_string_append( wfdbp, wavedir->str );
  setwfdb(wfdbp->str);

  /* setup gui, quit if unsuccessful */
  if (initialize_graphics(mode)) exit(1);

  /* Initialize the annotation table. */
  (void)read_anntab();

  /* Open the selected record (and annotation file, if specified). */
  if ( win_mode == 0 )
    if (record_init(record)) {
      if (annotator[0][0]) {
	af[0].name = annotator[0]; af[0].stat = WFDB_READ;
	nann = 1;
	annot_init();
      }
    }
  

  /* Indicate that the input annotation file must be saved before writing
     any edits. */
  savebackup = 1;

  /* If requested, set the start time. */
  if (start_string) {
    display_start_time = strtim(start_string);
    if (start_string[0] == '[') {
      if (display_start_time > 0L)
	/* a time before the beginning of the record was requested --
	   go to the beginning of the record instead */
	display_start_time = 0;
      else
	/* strtim converts a valid absolute time string into a negated
	   sample number -- change it to a positive sample number */
	display_start_time = -display_start_time;
    }
  }
  set_window_footer();

  /* If requested, start demonstration. */
//if (do_demo)
//  start_demo();

  /* Enter the main loop. */
  display_and_process_events();

  exit(0);
}




